@@ -124,6 +124,61 @@ GtkWidget *pidgin_create_imhtml(gboolean editable, GtkWidget **imhtml_ret, GtkWi
  */
 GtkWidget *pidgin_create_window(const char *title, guint border_width, const char *role, gboolean resizable);
 
+/**
+ * Creates a new dialog window
+ *
+ * @param title        The window title, or @c NULL
+ * @param border_width The window's desired border width
+ * @param role         A string indicating what the window is responsible for doing, or @c NULL
+ * @param resizable    Whether the window should be resizable (@c TRUE) or not (@c FALSE)
+ *
+ * @since 2.4.0
+ */
+GtkWidget *pidgin_create_dialog(const char *title, guint border_width, const char *role, gboolean resizable);
+
+/**
+ * Retrieves the main content box (vbox) from a pidgin dialog window
+ *
+ * @param dialog       The dialog window
+ * @param homogeneous  TRUE if all children are to be given equal space allotments. 
+ * @param spacing      the number of pixels to place by default between children
+ *
+ * @since 2.4.0
+ */
+GtkWidget *pidgin_dialog_get_vbox_with_properties(GtkDialog *dialog, gboolean homogeneous, gint spacing);
+
+/**
+ * Retrieves the main content box (vbox) from a pidgin dialog window
+ *
+ * @param dialog       The dialog window
+ *
+ * @since 2.4.0
+ */
+GtkWidget *pidgin_dialog_get_vbox(GtkDialog *dialog);
+
+/**
+ * Add a button to a dialog created by #pidgin_create_dialog.
+ *
+ * @param dialog         The dialog window
+ * @param label          The stock-id or the label for the button
+ * @param callback       The callback function for the button
+ * @param callbackdata   The user data for the callback function
+ *
+ * @return The created button.
+ * @since 2.4.0
+ */
+GtkWidget *pidgin_dialog_add_button(GtkDialog *dialog, const char *label,
+		GCallback callback, gpointer callbackdata);
+
+/**
+ * Retrieves the action area (button box) from a pidgin dialog window
+ *
+ * @param dialog       The dialog window
+ *
+ * @since 2.4.0
+ */
+GtkWidget *pidgin_dialog_get_action_area(GtkDialog *dialog);
+
 /**
  * Toggles the sensitivity of a widget.
  *
@@ -132,35 +187,6 @@ GtkWidget *pidgin_create_window(const char *title, guint border_width, const cha
  */
 void pidgin_toggle_sensitive(GtkWidget *widget, GtkWidget *to_toggle);
 
-#ifdef USE_HILDON
-/**
- * Places a widget inside a GtkScrolledWindow. The function is defined as a pass-through so as to be able to
- * accomodate different modifications to a container depending on need.
- *
- * In Hildon's case, its functionality is twofold:
- *
- * 1. It will at least place container inside a GtkScrolledWindow (with policy set to @c GTK_POLICY_AUTOMATIC)
- *    and return the GtkScrolledWindow.
- *
- * 2. If toplevel is given, then it attempts to deal with the bug whereby a GtkScrolledWindow
- *    with @c GTK_POLICY_AUTOMATIC, when placed inside a non-resizable GtkWindow (such as a GtkDialog), tends to
- *    become squashed to its minimum size. This function compensates by initially setting the policy to
- *    @c GTK_POLICY_NEVER, and overriding the GtkScrolledWindow size request to include the size of the scrollbars
- *    and viewport borders. Upon showing the GtkScrolledWindow, the policy is re-set to @c GTK_POLICY_AUTOMATIC.
- *
- *    This will only work on Maemo, because its window manager ensures that no portion of a dialog is ever shown
- *    off-screen, whereas desktop window managers will give the dialog all the size it asks for, even if that means
- *    that it won't fit on the screen.
- *
- * @param container The container to make scrollable.
- * @param toplevel  The toplevel widget (non-resizable window or dialog), or @c NULL
- * @return a GtkScrolledWindow containing a GtkViewport with the widget passed in.
- */
-GtkWidget *pidgin_make_container_scrollable(GtkWidget *container, GtkWidget *toplevel);
-#else /* !USE_HILDON */
-# define pidgin_make_container_scrollable(container,toplevel) container
-#endif /* USE_HILDON */
-
 /**
  * Checks if text has been entered into a GtkTextEntry widget.  If 
  * so, the GTK_RESPONSE_OK on the given dialog is set to TRUE.  
@@ -352,16 +378,20 @@ void pidgin_setup_screenname_autocomplete_with_filter(GtkWidget *entry, GtkWidge
 gboolean pidgin_screenname_autocomplete_default_filter(const PidginBuddyCompletionEntry *completion_entry, gpointer all_accounts);
 
 /**
- * @deprecated
  * Add autocompletion of screenames to an entry.
- * The usage of this function is deprecated. For new code, use the equivalent:
- * pidgin_setup_screenname_autocomplete_with_filter(entry, optmenu, pidgin_screenname_autocomplete_default_filter, GINT_TO_POINTER(all))
+ *
+ * @deprecated
+ *   For new code, use the equivalent:
+ *   #pidgin_setup_screenname_autocomplete_with_filter(@a entry, @a optmenu,
+ *   #pidgin_screenname_autocomplete_default_filter, <tt>GINT_TO_POINTER(@a
+ *   all)</tt>)
  *
  * @param entry     The GtkEntry on which to setup autocomplete.
- * @param optmenu   A menu for accounts, returned by pidgin_account_option_menu_new().
- *                  If @a optmenu is not @c NULL, it'll be updated when a screenname is chosen
- *                  from the autocomplete list.
- * @param all       Whether to include screennames from disconnected accounts. 
+ * @param optmenu   A menu for accounts, returned by
+ *                  pidgin_account_option_menu_new().  If @a optmenu is not @c
+ *                  NULL, it'll be updated when a screenname is chosen from the
+ *                  autocomplete list.
+ * @param all       Whether to include screennames from disconnected accounts.
  */
 void pidgin_setup_screenname_autocomplete(GtkWidget *entry, GtkWidget *optmenu, gboolean all);
 
@@ -484,7 +514,7 @@ void pidgin_menu_position_func_helper(GtkMenu *menu, gint *x, gint *y,
 
 /**
  * A valid GtkMenuPositionFunc.  This is used to determine where 
- * to draw context menu's when the menu is activated with the 
+ * to draw context menus when the menu is activated with the 
  * keyboard (shift+F10).  If the menu is activated with the mouse, 
  * then you should just use GTK's built-in position function, 
  * because it does a better job of positioning the menu.
@@ -758,5 +788,30 @@ const char *pidgin_text_combo_box_entry_get_text(GtkWidget *widget);
  */
 void pidgin_text_combo_box_entry_set_text(GtkWidget *widget, const char *text);
 
+/**
+ * Automatically make a window transient to a suitable parent window.
+ *
+ * @param window    The window to make transient.
+ *
+ * @return  Whether the window was made transient or not.
+ * @since 2.4.0
+ */
+gboolean pidgin_auto_parent_window(GtkWidget *window);
+
+/**
+ * Add a labelled widget to a GtkVBox
+ *
+ * @param vbox         The GtkVBox to add the widget to.
+ * @param widget_label The label to give the widget, can be @c NULL.
+ * @param sg           The GtkSizeGroup to add the label to, can be @c NULL.
+ * @param widget       The GtkWidget to add.
+ * @param expand       Whether to expand the widget horizontally.
+ * @param p_label      Place to store a pointer to the GtkLabel, or @c NULL if you don't care.
+ *
+ * @return  A GtkHBox already added to the GtkVBox containing the GtkLabel and the GtkWidget.
+ * @since 2.4.0
+ */
+GtkWidget *pidgin_add_widget_to_vbox(GtkBox *vbox, const char *widget_label, GtkSizeGroup *sg, GtkWidget *widget, gboolean expand, GtkWidget **p_label);
+
 #endif /* _PIDGINUTILS_H_ */
 
